#include <glib/gi18n.h>
#include <gio/gunixmounts.h>
#include <locale.h>
#include "thunar-encryption.h"
#include "config.h"

static void thunarx_encryption_menu_provider_init (ThunarxMenuProviderIface *iface);
static GList *thunarx_encryption_get_file_actions (ThunarxMenuProvider *provider, GtkWidget *window, GList *files);
static GList *thunarx_encryption_get_folder_actions (ThunarxMenuProvider *provider, GtkWidget *window, ThunarxFileInfo *folder);
static void thunarx_encryption_activated (ThunarxMenuItem *menuitem, gpointer *data);

struct _ThunarxEncriptionClass
{
	GObjectClass __parent__;
};

struct _ThunarxEncription
{
	GObject __parent__;
};


THUNARX_DEFINE_TYPE_WITH_CODE (ThunarxEncription, thunarx_encryption, G_TYPE_OBJECT,
	THUNARX_IMPLEMENT_INTERFACE (THUNARX_TYPE_MENU_PROVIDER, thunarx_encryption_menu_provider_init));

static void thunarx_encryption_class_init (ThunarxEncriptionClass *cls) {}

static void thunarx_encryption_init (ThunarxEncription *encryption) {}


static void thunarx_encryption_menu_provider_init (ThunarxMenuProviderIface *iface) {
	iface->get_file_menu_items = thunarx_encryption_get_file_actions;
	iface->get_folder_menu_items = thunarx_encryption_get_folder_actions;
}


static GList *thunarx_encryption_get_file_actions (ThunarxMenuProvider *provider, GtkWidget *window, GList *files) {
	if (G_LIKELY (files != NULL && files->next == NULL && thunarx_file_info_is_directory (files->data))) {
		return thunarx_encryption_get_folder_actions (provider, window, files->data);
	} else {
		return NULL;
	}
}


static GList* thunarx_encryption_get_folder_actions (ThunarxMenuProvider *provider, GtkWidget *window, ThunarxFileInfo *file) {
	setlocale (LC_ALL, "");
	bindtextdomain (PROJECT_NAME, LOCALE_DIR);
	bind_textdomain_codeset (PROJECT_NAME, "UTF-8");
	textdomain (PROJECT_NAME);
	
	ThunarxMenuItem *menu_mount = thunarx_menu_item_new ("ThunarxEncription::open", gettext ("Open encripted folder"), gettext ("Open (mount) encripted folder"), "encripted-folder");
	ThunarxMenuItem *menu_encript = thunarx_menu_item_new ("ThunarxEncription::crypt", gettext ("Encript folder"), gettext ("Encript empty folder"), "encripted-folder-new");
	ThunarxMenuItem *menu_umount = thunarx_menu_item_new ("ThunarxEncription::close", gettext ("Close encripted folder"), gettext ("Close (umount) encripted folder"), "encripted-folder");
	ThunarxMenuItem *menu_item = NULL;
	
	GFile *folder = thunarx_file_info_get_location (file);
	if (is_initialized (folder)) {
		if (is_mounted (folder)) {
			menu_item = menu_umount;
		} else {
			menu_item = menu_mount;
		}
	} else if (is_empty (folder)) {
		menu_item = menu_encript;
	}
	gchar *path = g_file_get_path (folder);
	g_object_unref (folder);
	
	if (menu_item != NULL) {
		g_signal_connect (G_OBJECT (menu_item), "activate", G_CALLBACK (thunarx_encryption_activated), window);
		g_object_set_data_full (G_OBJECT (menu_item), "path", path, g_free);
		return g_list_prepend (NULL, menu_item);
	} else {
		g_free (path);
		return NULL;
	}
}

gboolean is_mounted (GFile *folder) {
	gchar *path = g_file_get_basename (folder);
	gboolean mounted = FALSE;
	GVolumeMonitor *vm = g_volume_monitor_get ();
	GList *mounts = g_volume_monitor_get_mounts (vm);
	for (GList *current = mounts; !mounted && current != NULL; current = current->next) {
		gchar *_path = g_mount_get_name (current->data);
		if (g_strcmp0 (path, _path) == 0) mounted = TRUE;
		g_free (_path);
	}
	g_list_free_full (mounts, g_object_unref);
	g_object_unref (vm);
	g_free (path);
	return mounted;
}

gboolean is_initialized (GFile *folder) {
	gchar *path = g_file_get_path (folder);
	gchar *cryfs = g_strdup_printf ("%s/cryfs.config", path);
	gchar *encfs = g_strdup_printf ("%s/.encfs6.xml", path);
	gboolean initialized = g_file_test (cryfs, G_FILE_TEST_EXISTS) || g_file_test (encfs, G_FILE_TEST_EXISTS);
	g_free (encfs);
	g_free (cryfs);
	g_free (path);
	return initialized;
}

gboolean is_empty (GFile *folder) {
	gboolean empty = TRUE;
	GFileEnumerator *children = g_file_enumerate_children (folder, G_FILE_ATTRIBUTE_STANDARD_NAME, G_FILE_QUERY_INFO_NONE, NULL, NULL);
	GFileInfo *child = g_file_enumerator_next_file(children, NULL, NULL);
	if (child != NULL) {
		empty = FALSE;
		g_object_unref (child);
	}
	g_object_unref (children);
	return empty;
}

static void thunarx_encryption_activated (ThunarxMenuItem *menuitem, gpointer *data) {
	const gchar *path = g_object_get_data (G_OBJECT (menuitem), "path");
	gchar *commandline = g_strdup_printf ("gcryfolder.sh \"%s\"", path);
	g_spawn_command_line_async (commandline, NULL);
	g_free (commandline);
}


